@@ -220,7 +220,7 @@ class GaussianDiffusion(nn.Module):
         register_buffer('posterior_mean_coef1', betas * torch.sqrt(alphas_cumprod_prev) / (1. - alphas_cumprod))
         register_buffer('posterior_mean_coef2', (1. - alphas_cumprod_prev) * torch.sqrt(alphas) / (1. - alphas_cumprod))
 
-    def get_times(self, noise_level):
+    def get_times(self, batch_size, noise_level):
         device = self.betas.device
         return torch.full((batch_size,), int(self.num_timesteps * noise_level), device = device, dtype = torch.long)
 
@@ -267,7 +267,7 @@ class GaussianDiffusionContinuousTimes(GaussianDiffusion):
         super().__init__()
         raise NotImplementedError
 
-    def get_times(self, noise_level):
+    def get_times(self, batch_size, noise_level):
         device = self.betas.device
         return torch.full((batch_size,), noise_level, device = device, dtype = torch.long)
 
@@ -668,6 +668,7 @@ class Unet(nn.Module):
         lowres_cond = False, # for cascading diffusion - https://cascaded-diffusion.github.io/
         layer_attns = True,
         attend_at_middle = True, # whether to have a layer of attention at the bottleneck (can turn off for higher resolution in cascading DDPM, before bringing in efficient attention)
+        layer_cross_attns = True,
         cond_on_text = True,
         max_text_len = 256,
         init_dim = None,
@@ -759,13 +760,17 @@ class Unet(nn.Module):
 
         attn_kwargs = dict(heads = attn_heads, dim_head = attn_dim_head)
 
+        num_layers = len(in_out)
+
         # resnet block klass
 
-        num_resnet_blocks = cast_tuple(num_resnet_blocks, len(in_out))
-        resnet_groups = cast_tuple(resnet_groups, len(in_out))
-        layer_attns = cast_tuple(layer_attns, len(in_out))
+        num_resnet_blocks = cast_tuple(num_resnet_blocks, num_layers)
+        resnet_groups = cast_tuple(resnet_groups, num_layers)
 
-        assert len(resnet_groups) == len(in_out)
+        layer_attns = cast_tuple(layer_attns, num_layers)
+        layer_cross_attns = cast_tuple(layer_cross_attns, num_layers)
+
+        assert all([layers == num_layers for layers in list(map(len, (resnet_groups, layer_attns, layer_cross_attns)))])
 
         # downsample klass
 
@@ -779,13 +784,15 @@ class Unet(nn.Module):
         self.ups = nn.ModuleList([])
         num_resolutions = len(in_out)
 
-        for ind, ((dim_in, dim_out), layer_num_resnet_blocks, groups, layer_attn) in enumerate(zip(in_out, num_resnet_blocks, resnet_groups, layer_attns)):
-            is_first = ind == 0
+        layer_params = [num_resnet_blocks, resnet_groups, layer_attns, layer_cross_attns]
+        reversed_layer_params = list(map(reversed, layer_params))
+
+        for ind, ((dim_in, dim_out), layer_num_resnet_blocks, groups, layer_attn, layer_cross_attn) in enumerate(zip(in_out, *layer_params)):
             is_last = ind >= (num_resolutions - 1)
-            layer_cond_dim = cond_dim if not is_first else None
+            layer_cond_dim = cond_dim if layer_cross_attn else None
 
             self.downs.append(nn.ModuleList([
-                ResnetBlock(dim_in, dim_out, time_cond_dim = time_cond_dim, groups = groups),
+                ResnetBlock(dim_in, dim_out, cond_dim = layer_cond_dim, time_cond_dim = time_cond_dim, groups = groups),
                 nn.ModuleList([ResnetBlock(dim_out, dim_out, groups = groups) for _ in range(layer_num_resnet_blocks)]),
                 TransformerBlock(dim = dim_out, heads = attn_heads, dim_head = attn_dim_head, ff_mult = ff_mult) if layer_attn else nn.Identity(),
                 downsample_klass(dim_out) if not is_last else nn.Identity()
@@ -797,9 +804,8 @@ class Unet(nn.Module):
         self.mid_attn = EinopsToAndFrom('b c h w', 'b (h w) c', Residual(Attention(mid_dim, **attn_kwargs))) if attend_at_middle else None
         self.mid_block2 = ResnetBlock(mid_dim, mid_dim, cond_dim = cond_dim, time_cond_dim = time_cond_dim, groups = resnet_groups[-1])
 
-        for ind, ((dim_in, dim_out), layer_num_resnet_blocks, groups, layer_attn) in enumerate(zip(reversed(in_out[1:]), reversed(num_resnet_blocks), reversed(resnet_groups), reversed(layer_attns))):
-            is_last = ind >= (num_resolutions - 2)
-            layer_cond_dim = cond_dim if not is_last else None
+        for ind, ((dim_in, dim_out), layer_num_resnet_blocks, groups, layer_attn, layer_cross_attn) in enumerate(zip(reversed(in_out[1:]), *reversed_layer_params)):
+            layer_cond_dim = cond_dim if layer_cross_attn else None
 
             self.ups.append(nn.ModuleList([
                 ResnetBlock(dim_out * 2, dim_in, cond_dim = layer_cond_dim, time_cond_dim = time_cond_dim, groups = groups),
@@ -978,11 +984,37 @@ class Unet(nn.Module):
 
         return self.final_conv(x)
 
-class SRUnet(Unet):
+# predefined unets, with configs lining up with hyperparameters in appendix of paper
+
+class BaseUnet64(Unet):
     def __init__(self, *args, **kwargs):
-        kwargs.update({'layer_attns': False})
+        kwargs.update(dict(
+            num_resnet_blocks = 3,
+            layer_attns = (False, True, True, True),
+            layer_cross_attns = (False, True, True, True)
+        ))
         super().__init__(*args, **kwargs)
 
+class SRUnet256(Unet):
+    def __init__(self, *args, **kwargs):
+        kwargs.update(dict(
+            num_resnet_blocks = (2, 4, 8, 8),
+            layer_attns = (False, False, False, True),
+            layer_cross_attns = (False, False, False, True)
+        ))
+        super().__init__(*args, **kwargs)
+
+class SRUnet1024(Unet):
+    def __init__(self, *args, **kwargs):
+        kwargs.update(dict(
+            num_resnet_blocks = (2, 4, 8, 8),
+            layer_attns = False,
+            layer_cross_attns = (False, False, False, True)
+        ))
+        super().__init__(*args, **kwargs)
+
+# main imagen ddpm class, which is a cascading DDPM from Ho et al.
+
 class Imagen(nn.Module):
     def __init__(
         self,
@@ -1237,7 +1269,7 @@ class Imagen(nn.Module):
                 shape = (batch_size, channel, image_size, image_size)
 
                 if unet.lowres_cond:
-                    lowres_noise_times = noise_scheduler.get_times(lowres_sample_noise_level)
+                    lowres_noise_times = noise_scheduler.get_times(batch_size, lowres_sample_noise_level)
 
                     lowres_cond_img = resize_image_to(img, image_size)
                     lowres_cond_img = noise_scheduler.q_sample(x_start = lowres_cond_img, t = lowres_noise_times, noise = torch.randn_like(lowres_cond_img))
